function [Fvalues,solutions] = tchebycheff_MOP(objectives,constraints,bounds,N)

    rho=0.01;
    params.OutputFlag=0;
    params.FeasibilityTol=1e-9;
    obj_names = fieldnames(objectives);
    K=size(obj_names,1);
    [m,n]=size(constraints.A);
    
    objcoeffs=zeros(K,length(objectives.(obj_names{1}).c));
    for k=1:K
        objcoeffs(k,:)=objectives.(obj_names{k}).c;
    end
    
    % Construct payoff matrix by optimizing each objective separately
    singleobjsolutions=zeros(n,K);
    linearObjCoefs=zeros(K,n);
    Qmatrix=sparse(n,n);
    for k=1:K
        model=struct();
        model.obj=objectives.(obj_names{k}).c;
        linearObjCoefs(k,:)=model.obj;
        if isfield(objectives.(obj_names{k}),'Q')
            model.Q=sparse(objectives.(obj_names{k}).Q);
            Qmatrix=Qmatrix+model.Q;
        end
        model.A=sparse(constraints.A);
        model.rhs=constraints.rhs;
        model.modelsense='max';
        model.sense=constraints.sense;
        model.lb=bounds(:,1);
        model.ub=bounds(:,2);
        result=gurobi(model,params);
        if ~isfield(objectives.(obj_names{k}),'Q')
            model_lexicographic=struct;
            model_lexicographic.A=[model.A;model.obj];
            model_lexicographic.sense=model.sense;
            model_lexicographic.sense(end+1)='=';
            model_lexicographic.rhs=model.rhs;
            model_lexicographic.lb=model.lb;
            model_lexicographic.ub=model.ub;
            model_lexicographic.rhs(end+1)=result.objval;
            model_lexicographic.modelsense='max';
            objstosum=setdiff([1:K],k);
            if length(objstosum)>1
                model_lexicographic.obj=sum(objcoeffs(objstosum,:));
            else
                model_lexicographic.obj=objcoeffs(objstosum,:);
            end
            model_lexicographic.Q=sparse(size(model_lexicographic.A,2),size(model_lexicographic.A,2));
            for k1=1:K
                if k1~=k & isfield(objectives.(obj_names{k1}),'Q')
                    model_lexicographic.Q=model_lexicographic.Q+sparse(objectives.(obj_names{k1}).Q);
                end
            end                    
            result=gurobi(model_lexicographic,params);
        else
            model_lexicographic=struct;
            model_lexicographic.A=model.A;            
            model_lexicographic.quadcon(1).Qc=-sparse(objectives.(obj_names{k}).Q);
            model_lexicographic.quadcon(1).q=zeros(size(model_lexicographic.A,2),1);
            model_lexicographic.quadcon(1).rhs=-result.objval;
            model_lexicographic.rhs=model.rhs;
            model_lexicographic.sense=model.sense;            
            model_lexicographic.lb=model.lb;
            model_lexicographic.ub=model.ub;
            model_lexicographic.modelsense='max';
            objstosum=setdiff([1:K],k);
            if length(objstosum)>1
                model_lexicographic.obj=sum(objcoeffs(objstosum,:));
            else
                model_lexicographic.obj=objcoeffs(objstosum,:);
            end
            model_lexicographic.Q=sparse(size(model_lexicographic.A,2),size(model_lexicographic.A,2));
            for k1=1:K
                if k1~=k & isfield(objectives.(obj_names{k1}),'Q')
                    model_lexicographic.Q=model_lexicographic.Q+sparse(objectives.(obj_names{k1}).Q);
                end
            end                    
            result=gurobi(model_lexicographic,params);            
        end
        singleobjsolutions(:,k)=result.x;
        payoff(k,:)=calc_objectives(objectives,result.x);
    end       

    % Find ideal, nadir, utopian and the objectives' normalization factors
    ideal=max(payoff);
    nadir=min(payoff);
    utopian=ideal+0.01*abs(ideal);
    norm_factors=100*(1./(ideal-nadir))./sum(1./(ideal-nadir));

    %Generate 10*N random weights for the objectives
    for i=1:10*N
        if rand()<0.4
            u=unifrnd(0,1,K,1);            
        else
        	u=0.01*(-log(unifrnd(0,1,K,1))).^(1/0.3);
        end
        fweights(:,i)=u/sum(u);
    end
    %Filter the weights down to N weight vectors
    %[selected_wvectors,test]=KennardStone(fweights',2*N);
    selected_wvectors=filter_weights(fweights',2*N);
    fweights=fweights(:,selected_wvectors);

    % Generate 2*N efficient solutions with a Chebyshev (Tchebycheff) scalarization function
    model.A=[sparse(constraints.A) zeros(m,1)];
    model.lb=[bounds(:,1);0];
    model.ub=[bounds(:,2);inf];

    Fvalues=zeros(K,N);
    solutions=zeros(n,N);
    parfor i=1:2*N
        qcon=1;
        modeltmp=struct();
        modeltmp=model;
        modeltmp.modelsense='min';
        modeltmp.obj=[-rho*sum(linearObjCoefs) 1];
        if sum(sum(abs(Qmatrix)))>0
            modeltmp.Q=-rho*sparse(blkdiag(Qmatrix,0));
        end
        for k=1:K
            if isfield(objectives.(obj_names{k}),'Q')
                modeltmp.quadcon(qcon).Qc=sparse(-norm_factors(k)*fweights(k,i)*blkdiag(objectives.(obj_names{k}).Q,0));
                modeltmp.quadcon(qcon).q=-[norm_factors(k)*fweights(k,i)*objectives.(obj_names{k}).c 1];
                modeltmp.quadcon(qcon).rhs=-fweights(k,i)*utopian(k)*norm_factors(k);
                qcon=qcon+1;
            else
                modeltmp.A=[modeltmp.A;[norm_factors(k)*fweights(k,i)*objectives.(obj_names{k}).c 1]];
                modeltmp.rhs=[modeltmp.rhs;fweights(k,i)*utopian(k)*norm_factors(k)];
                modeltmp.sense(end+1)='>';
            end
        end
        result=gurobi(modeltmp,params);
        Fvalues(:,i)=(calc_objectives(objectives,result.x(1:n)))';
        solutions(:,i)=result.x(1:n);
    end
    Fvalues=round([Fvalues payoff']*1000)/1000;    
    solutions=[solutions singleobjsolutions];
    [Fvalues,ia,ic] = unique(Fvalues','rows');
    solutions=solutions(:,ia);
    [mx,ids]=max(Fvalues);
    Ftmp=Fvalues(setdiff([1:size(Fvalues,1)],ids),:);
    soltmp=solutions(:,setdiff([1:size(Fvalues,1)],ids));
    selected_fvectors=filter_weights(Ftmp,N-K);
    solutions=[solutions(:,ids) soltmp(:,selected_fvectors)];
    Fvalues=[Fvalues(ids,:);Ftmp(selected_fvectors,:)];
    Fvalues=Fvalues';
end

% function to calculate objectives
function F=calc_objectives(objectives,x)
    obj_names = fieldnames(objectives);
    K=size(obj_names,1);
    for k=1:K
        if isfield(objectives.(obj_names{k}),'Q')
            F(1,k)=objectives.(obj_names{k}).c*x+x'*objectives.(obj_names{k}).Q*x;
        else
            F(1,k)=objectives.(obj_names{k}).c*x;
        end
    end    
end